#include <reg167.h>
#include <CS8900.h>
#include <string.h>

#define IP1	 	192
#define IP2     168
#define IP3     1
#define IP4     2

// 48 bit IEEE OUI (Organizationally Unique Identifier)
#define MAC1    0x00
#define MAC2    0x00
#define MAC3    0x00
#define MAC4    0x12
#define MAC5    0x34
#define MAC6    0x56



BOOL CheckIOInterface(void) {
  WORD pic, rev, iobadr;
  
// check product identification code and revision number
  Ether.IO.PPP = cProductIdentificationCode | cAutoIncrement;
  pic = Ether.IO.PPD;
  rev = Ether.IO.PPD;
  if (pic != 0x630E) return(FALSE);     // chip not accessible
//  if (rev != 0x0900) return(FALSE);     // chip not accessible (chip version may vary)

  Ether.IO.PPP = cIOBaseAddress;
  iobadr = Ether.IO.PPD;
  if (iobadr != 0x0300) return (FALSE); // must be 0x0300

// switch on the LINK LED
  Ether.IO.PPP = cSelfCTL;
  Ether.IO.PPD |= 0x5000;

  return(TRUE);
}

BOOL EnableMemInterface(void) {
  DWORD volatile pic;
  WORD  volatile v1, v2;

  Ether.IO.PPP = cMemBaseAddress | cAutoIncrement;
  v1 = Ether.IO.PPP;    // load low word of address
  Ether.IO.PPD = 0x0000;    // load low word of address
  Ether.IO.PPD = 0x0000;    // load high word of address

  v1 = Ether.IO.PPP;    // load low word of address
  Ether.IO.PPP = cMemBaseAddress | cAutoIncrement;
  v1 = Ether.IO.PPD;    // load low word of address
  v2 = Ether.IO.PPD;    // load high word of address

  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;

  Ether.IO.PPP = cBusCTL;   // Select Bus Contol Register Reg17
  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;
  Ether.IO.PPD |= 0x0400;   // Set MemoryE Bit

  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;
  pic = Ether.ProductIdentificationCode;
  Ether.IO.PPP = cMemBaseAddress | cAutoIncrement;
  v1 = Ether.IO.PPD;    // load low word of address
  v2 = Ether.IO.PPD;    // load high word of address
  if ((pic & 0x0000ffff) != 0x0000630E)  // Read Product Ident Code as test (chip version may vary)
    return (FALSE);

  return (TRUE);             // Return successful
}

BOOL ResetChip(void) { 
  WORD i, rev, pic;

  Ether.IO.PPP = cSelfCTL;
  Ether.IO.PPD = SELF_CTL_RESET; //issue a reset to the chip
  i = 0;
  while(Ether.IO.PPD & SELF_CTL_RESET) i++;      // wait until reset done
  i = 0;
  Ether.IO.PPP = cSelfST;
  while(!(Ether.IO.PPD & SELF_ST_INIT_DONE)) i++; // wait until INITD bit set
  Ether.IO.PPP = cProductIdentificationCode | cAutoIncrement;
  pic = Ether.IO.PPD;
  rev = Ether.IO.PPD;
  if (pic != 0x630E) return(FALSE);     // chip not accessible
//  if (rev != 0x0900) return(FALSE);     // chip not accessible (chip version may vary)

  return(TRUE);
}



// Initializes the chip 

BOOL InitChip(void) {

  Ether.LineCTL = LINE_CTL_10BASET;  // set to 10BaseT
  Ether.TestCTL = TEST_CTL_FDX /*| TEST_CTL_ENDEC_LP*/; // set to full duplex  ???  Loopback??
  Ether.RxCFG   = RX_CFG_RX_OK_IE;   // enable RxOK interrupt
  Ether.RxCTL   = (RX_CTL_RX_OK_A | RX_CTL_IND_A | RX_CTL_BCAST_A);
  Ether.TxCFG   = TX_CFG_ALL_IE;

// Important: The IA needs to be byte revered IA=aa:bb:cc:dd:ee:ff

  Ether.IndividualAddress[0] = MAC1;
  Ether.IndividualAddress[1] = MAC2;
  Ether.IndividualAddress[2] = MAC3;
  Ether.IndividualAddress[3] = MAC4;
  Ether.IndividualAddress[4] = MAC5;
  Ether.IndividualAddress[5] = MAC6;

//  Ether.IndAddr = (MAC2<<8) | MAC1;  // 0xbbaa Write
//         ppWR   ppIndAddr+2,(MAC4<<8|MAC3)  ;0xddcc  out
//         ppWR   ppIndAddr+4,(MAC6<<8|MAC5)  ;0xffee   48 bit IA
  Ether.InterruptNumber = 0x00;              //INT is on INTRQ0

  Ether.BusCTL  |= BUS_CTL_INT_ENBL;         // enable interrupt
  Ether.LineCTL |= LINE_CTL_RX_ON | LINE_CTL_TX_ON; // set SerRxOn and SerTxOn

  return(TRUE);
}

#if 0
BOOL CheckMemInterface(void) {
  WORD volatile *pWord, i;
  
  for (i = 0; i < 0x300; i++) {
    Ether.TransmitFrameLocation[i] = 0xa5a5;  // write test pattern
  }
  for (i = 0; i < 0x300; i++) {
    if (Ether.TransmitFrameLocation[i] != 0xa5a5) return (FALSE); // check test pattern
  }

//  pWord
  return(TRUE);
}
#endif


void error(void) {

  while(1);
}

#if 0          // Not completed yet
typedef struct  {
  WORD  RXStatus;
  WORD  RXLength;
  BYTE  Content[1532];
} RXFRAME;

RXFRAME RxFrames[3];
RXFRAME *RxBuffer = &RxFrames[0];

void EtherInt(void) interrupt 0x33 {
  WORD IRQSource, RxStatus, RxLength;

  IRQSource = Ether.ISQ;    // Read Interrupt Service Queue
  switch(IRQSource & 0x003F) {
    case 0x04:              // Receive Interrupt occured
// IRQSource contains the RxEvent Register contents
// Do some check on IRQSource to be sure that the interrupt signals a new frame
      RxStatus = Ether.RxStatus;
      RxBuffer->RxStatus = RxStatus;
      RxLength = Ether.RxLength;
      RxBuffer->RxLength = RxLength;
      RxLength = (RxLength + 1) & 0x00FE;
      xmemcpy(&RxBuffer->Content, &Ether.ReceiveFrameLocation[0], RxLength); // Check if memcpy performs word transfer!
      if (Ether.ReceiveFrameLocation[16] == 0x08) {
        if (Ether.ReceiveFrameLocation[17] == 0x06) {        // ARP
        } else if (Ether.ReceiveFrameLocation[17] == 0x00) { // IP
        }
      }
      break;

    case 0x08:              // Transmit Interrupt occured
// IRQSource contains the TxEvent Register contents
      break;

    case 0x0C:              // Buffer Interrupt occured
// IRQSource contains the BufEvent Register contents
      break;

    case 0x10:              // Receive Buffer Miss Interrupt occured
// IRQSource contains the RxMISS Register contents
      break;

    case 0x12:              // Transmit Collision Counter Interrupt occured
// IRQSource contains the TxCOLL Register contents
      break;

    default:
// This should never occur
      break;

}
#endif

/******************************************************************************/
/***************************      MAIN PROGRAM      ***************************/
/******************************************************************************/
void main ( void )  {                          /* main entry for program      */

  if (CheckIOInterface() == FALSE) error();

  if (ResetChip() == FALSE) error();

  if (EnableMemInterface() == FALSE) error();

  if (InitChip() == FALSE) error();

  while(1);
}


 
